@@ -35,23 +35,10 @@
 
 static std::string doubleToString(const double value)
 {
-  std::ostringstream tempStream;
-  tempStream << value;
-#ifndef __ANDROID__
-  std::string decimalPoint(localeconv()->decimal_point);
-#else
-  std::string decimalPoint(".");
-#endif
-  if ((decimalPoint.size() == 0) || (decimalPoint == "."))
-    return tempStream.str();
-  std::string stringValue(tempStream.str());
-  if (!stringValue.empty())
-  {
-    std::string::size_type pos;
-    while ((pos = stringValue.find(decimalPoint)) != std::string::npos)
-      stringValue.replace(pos,decimalPoint.size(),".");
-  }
-  return stringValue;
+  WPXProperty *prop = WPXPropertyFactory::newDoubleProp(value);
+  std::string retVal = prop->getStr().cstr();
+  delete prop;
+  return retVal;
 }
 
 static unsigned stringToColour(const ::WPXString &s)
@@ -336,7 +323,9 @@ void libvisio::VSDSVGGenerator::setStyle(const ::WPXPropertyList &propList, cons
 
 void libvisio::VSDSVGGenerator::startLayer(const ::WPXPropertyList &propList)
 {
-  m_outputSink << "<svg:g id=\"Layer" << propList["svg:id"]->getInt() << "\"";
+  m_outputSink << "<svg:g";
+  if (propList["svg:id"])
+    m_outputSink << " id=\"Layer" << propList["svg:id"]->getInt() << "\"";
   if (propList["svg:fill-rule"])
     m_outputSink << " fill-rule=\"" << propList["svg:fill-rule"]->getStr().cstr() << "\"";
   m_outputSink << " >\n";
@@ -449,7 +438,7 @@ void libvisio::VSDSVGGenerator::drawPath(const ::WPXPropertyListVector &path)
       m_outputSink << (propList["libwpg:sweep"] ? propList["libwpg:sweep"]->getInt() : 1) << " ";
       m_outputSink << doubleToString(72*(propList["svg:x"]->getDouble())) << "," << doubleToString(72*(propList["svg:y"]->getDouble()));
     }
-    else if ((i >= path.count()-1 && i > 2) && propList["libwpg:path-action"] && propList["libwpg:path-action"]->getStr() == "Z" )
+    else if (propList["libwpg:path-action"] && propList["libwpg:path-action"]->getStr() == "Z")
     {
       isClosed = true;
       m_outputSink << "\nZ";
@@ -505,7 +494,6 @@ void libvisio::VSDSVGGenerator::startTextObject(const ::WPXPropertyList &propLis
 {
   double x = 0.0;
   double y = 0.0;
-  double width = 0.0;
   double height = 0.0;
   m_outputSink << "<svg:text ";
   if (propList["svg:x"] && propList["svg:y"])
@@ -519,7 +507,7 @@ void libvisio::VSDSVGGenerator::startTextObject(const ::WPXPropertyList &propLis
 
   if (propList["svg:width"])
   {
-    width = propList["svg:width"]->getDouble();
+    double width = propList["svg:width"]->getDouble();
     xmiddle += width / 2.0;
   }
 
