@@ -168,6 +168,14 @@ void libvisio::VSDXMLParserBase::readGeometry(xmlTextReaderPtr reader)
       if (XML_READER_TYPE_ELEMENT == tokenType)
         readRelQuadBezTo(reader);
       break;
+    case XML_SPLINESTART:
+      if (XML_READER_TYPE_ELEMENT == tokenType)
+        readSplineStart(reader);
+      break;
+    case XML_SPLINEKNOT:
+      if (XML_READER_TYPE_ELEMENT == tokenType)
+        readSplineKnot(reader);
+      break;
     default:
       break;
     }
@@ -1010,6 +1018,7 @@ void libvisio::VSDXMLParserBase::readShape(xmlTextReaderPtr reader)
       m_shape.m_charList = tmpShape->m_charList;
       m_shape.m_paraList = tmpShape->m_paraList;
       m_shape.m_text = tmpShape->m_text;
+      m_shape.m_misc = tmpShape->m_misc;
     }
   }
 
@@ -1097,6 +1106,7 @@ void libvisio::VSDXMLParserBase::readPage(xmlTextReaderPtr reader)
   xmlChar *id = xmlTextReaderGetAttribute(reader, BAD_CAST("ID"));
   xmlChar *bgndPage = xmlTextReaderGetAttribute(reader, BAD_CAST("BackPage"));
   xmlChar *background = xmlTextReaderGetAttribute(reader, BAD_CAST("Background"));
+  xmlChar *pageName = xmlTextReaderGetAttribute(reader, BAD_CAST("NameU"));
   if (id)
   {
     unsigned nId = (unsigned)xmlStringToLong(id);
@@ -1104,7 +1114,7 @@ void libvisio::VSDXMLParserBase::readPage(xmlTextReaderPtr reader)
     bool isBackgroundPage = background ? xmlStringToBool(background) : false;
     m_isPageStarted = true;
     m_collector->startPage(nId);
-    m_collector->collectPage(nId, (unsigned)getElementDepth(reader), backgroundPageID, isBackgroundPage);
+    m_collector->collectPage(nId, (unsigned)getElementDepth(reader), backgroundPageID, isBackgroundPage, pageName ? VSDName(WPXBinaryData(pageName, xmlStrlen(pageName)), VSD_TEXT_UTF8) : VSDName());
   }
   if (id)
     xmlFree(id);
@@ -1112,6 +1122,8 @@ void libvisio::VSDXMLParserBase::readPage(xmlTextReaderPtr reader)
     xmlFree(bgndPage);
   if (background)
     xmlFree(background);
+  if (pageName)
+    xmlFree(pageName);
 }
 
 void libvisio::VSDXMLParserBase::readText(xmlTextReaderPtr reader)
@@ -1585,7 +1597,7 @@ void libvisio::VSDXMLParserBase::readSplineStart(xmlTextReaderPtr reader)
       break;
     }
   }
-  while (((XML_ELLIPTICALARCTO != tokenId && XML_ROW != tokenId) || XML_READER_TYPE_END_ELEMENT != tokenType) && 1 == ret);
+  while (((XML_SPLINESTART != tokenId && XML_ROW != tokenId) || XML_READER_TYPE_END_ELEMENT != tokenType) && 1 == ret);
   if (ret == 1)
     m_currentGeometryList->addSplineStart(ix, level, x, y, a, b, c, d);
 }
@@ -1676,6 +1688,8 @@ void libvisio::VSDXMLParserBase::readForeignData(xmlTextReaderPtr reader)
       m_shape.m_foreign->type = 2;
     else if (xmlStrEqual(foreignTypeString, BAD_CAST("EnhMetaFile")))
       m_shape.m_foreign->type = 4;
+    else if (xmlStrEqual(foreignTypeString, BAD_CAST("MetaFile")))
+      m_shape.m_foreign->type = 0;
     xmlFree(foreignTypeString);
   }
   xmlChar *foreignFormatString = xmlTextReaderGetAttribute(reader, BAD_CAST("CompressionType"));
@@ -1710,6 +1724,8 @@ void libvisio::VSDXMLParserBase::_flushShape()
 
   m_collector->collectXFormData(m_currentShapeLevel+2, m_shape.m_xform);
 
+  m_collector->collectMisc(m_currentShapeLevel+2, m_shape.m_misc);
+
   if (m_shape.m_txtxform)
     m_collector->collectTxtXForm(m_currentShapeLevel+2, *(m_shape.m_txtxform));
 
@@ -1760,7 +1776,7 @@ void libvisio::VSDXMLParserBase::_flushShape()
     }
   }
 
-  if (m_shape.m_foreign)
+  if (m_shape.m_foreign && m_shape.m_foreign->data.size())
     m_collector->collectForeignData(m_currentShapeLevel+1, m_shape.m_foreign->data);
 
   if (!m_shape.m_fields.empty())
